#This code is for the Dream-challenge acute myeloid leukemia data (see the reference Noren et al. 2016).
#Data can be downloaded for open research from the DREAM-website upon registration.
#Check that the dimension equals to (191,272) and remove the non-gene-expression variables.
data <- data[,c(42:272)]

#Install dependencies
library(matrixStats)
library(glmnet)


#Residual variance for the simulated phenotype replicates. The following residual variance was used in the simulation section to produce figure 2.
sigma1 = 1.1;
#Set the alpha1 and (and lambda if the cross-validation based value is not used) for the elastic net estimator when the bandwidth parameters are estimated.
alpha1 = 1/3; #lambda = 0.18

#The number of simulated replicates
nrep = 100


#a vector for the phenotype replicates
y1 <- matrix(0,nrow(data),nrep)
#a Vector for heritabilities over replications
heritvec1 <- rep(0,nrep)

####Simulated phenotype replicates with the average heritabilities approximately equal to 0.4#####

for(i in 1:nrep){
  
  expset <- data; expset <- as.matrix(expset)
  
  y1[,i] <- 0.45*expset[,45]+ 0.45*expset[,155] + 0.3*expset[,185] +expset[,21]*expset[,105]*expset[,207] + rnorm(191,0,sigma1)
  
  heritvec1[i] <- (var(y1[,i])-sigma1^2)/var(y1[,i])
  
}

#####Compute the average heritabilities over replicates.
mean(heritvec1)

#Empty vectors for the results (estimated inverse bandwidth parameters).
PHE <- matrix(0,ncol(data),nrep)
PHEres <- matrix(0,ncol(data),nrep)


##############THE PHE-METHOD WITHOUT THE RESIDUAL STEP##########################

for(i in 1:nrep){
  
  HEset <- cbind(y1[,i],expset)
  HEset <- HEset[order(HEset[,1]),]
  foo <- t(HEset)
  cols <- combn( ncol(foo), 2)
  Pairdiffs <- apply( cols, 2, function(z) rowDiffs(foo[,z]))
  HEset <- t(Pairdiffs)
  # Pairwise differences of phenotype values between all individuals
  HEpheno2 <- HEset[,1]
  # Pairwise differences of covariates between all individuals
  HEexpset <- HEset[,-1]
  # set gamma parameter equal to 0.2 in the powered exponential kernel
  v1 <- (abs(HEexpset))^0.2
  # Pseudo-explanatory variables
  pred1 <- (-v1);  
  
  # Inverse bandwidth parameters are estimated with the glmnet-function
  fit1=glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit1= cv.glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # Estimated bandwidth parameters with respect to the cross-validation based value presented in the paper
  a1 <- (abs(coef(fit1, s = 3*cv.fit1$lambda.1se))[-1])
  PHE[,i] <- as.vector(a1)
}



#########################WITH THE RESIDUAL STEP########################

for(i in 1:nrep){
  
  
  cv.fit <- cv.glmnet(expset, y1[,i], alpha = alpha1, maxit = 10000)
  fit=glmnet(expset, y1[,i], alpha = alpha1, maxit = 10000)
  Coefficients <- coef(fit, s = cv.fit$lambda.min)
  #NON-zero elements
  v <- expset %*% Coefficients[-1]
  residual <- y1[,i] - v
  
  
  HEset <- cbind(residual,expset)
  HEset <- HEset[order(HEset[,1]),]
  foo <- t(HEset)
  cols <- combn( ncol(foo), 2)
  pairDiffs <- apply( cols, 2, function(z) rowDiffs(foo[,z]))
  HEset <- t(pairDiffs)
  # Pairwise differences of phenotype values between all individuals
  HEpheno2 <- HEset[,1]
  # Pairwise differences of covariates between all individuals
  HEexpset <- HEset[,-1]
  # Different gamma parameters in the powered exponential kernel
  v1 <- (abs(HEexpset))^0.2
  # Creation of pseudo-explanatory variables
  pred1 <- (-v1); 
  # Inverse bandwidth parameters are estimated with glmnet function
  fit1=glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit1= cv.glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # The estimated bandwidth parameters with respect to the cross-validation based value presented in the paper
  a1 <- (abs(coef(fit1, s = 3*cv.fit1$lambda.1se))[-1])
  PHEres[,i] <- as.vector(a1)
}





par(mfrow = c(1,2))

plot(rowSums(PHE)/max(rowSums(PHE)), type = "l", main = "Without the residual step",xlab = "Genes", ylab ="Inverse bandwidths")
abline(v = c(21,105,207), col ="red", lty = 3)
abline(h = 0.22, lty = 3)
plot(rowSums(PHEres)/max(rowSums(PHEres)), type = "l", main = "With the residual step",xlab = "Genes", ylab ="Inverse bandwidths")
abline(v = c(21,105,207), col ="red", lty = 3)
abline(h = 0.22, lty = 3)

